@@ -38,7 +38,8 @@
  * properties of the 3GPP interface.
  *
  * The 3GPP interface is exposed whenever a modem has any of the 3GPP
- * capabilities (%MM_MODEM_CAPABILITY_GSM_UMTS, %MM_MODEM_CAPABILITY_LTE or %MM_MODEM_CAPABILITY_LTE_ADVANCED).
+ * capabilities (%MM_MODEM_CAPABILITY_GSM_UMTS, %MM_MODEM_CAPABILITY_LTE
+ * or %MM_MODEM_CAPABILITY_5GNR).
  */
 
 G_DEFINE_TYPE (MMModem3gpp, mm_modem_3gpp, MM_GDBUS_TYPE_MODEM3GPP_PROXY)
@@ -549,6 +550,23 @@ mm_modem_3gpp_network_free (MMModem3gppNetwork *network)
     g_slice_free (MMModem3gppNetwork, network);
 }
 
+static MMModem3gppNetwork *
+modem_3gpp_network_copy (MMModem3gppNetwork *network)
+{
+    MMModem3gppNetwork *network_copy;
+
+    network_copy = g_slice_new0 (MMModem3gppNetwork);
+    network_copy->availability      = network->availability;
+    network_copy->operator_long     = g_strdup (network->operator_long);
+    network_copy->operator_short    = g_strdup (network->operator_short);
+    network_copy->operator_code     = g_strdup (network->operator_code);
+    network_copy->access_technology = network->access_technology;
+
+    return network_copy;
+}
+
+G_DEFINE_BOXED_TYPE (MMModem3gppNetwork, mm_modem_3gpp_network, (GBoxedCopyFunc)modem_3gpp_network_copy, (GBoxedFreeFunc)mm_modem_3gpp_network_free)
+
 /**
  * mm_modem_3gpp_network_get_availability:
  * @network: A #MMModem3gppNetwork.
@@ -691,7 +709,7 @@ ensure_internal_initial_eps_bearer_settings (MMModem3gpp         *self,
             /* No need to clear this signal connection when freeing self */
             self->priv->initial_eps_bearer_settings_id =
                 g_signal_connect (self,
-                                  "notify::initial-eps-bearer-properties",
+                                  "notify::initial-eps-bearer-settings",
                                   G_CALLBACK (initial_eps_bearer_settings_updated),
                                   NULL);
         }
