@@ -13,6 +13,7 @@
  * Copyright (C) 2008 - 2009 Novell, Inc.
  * Copyright (C) 2009 - 2012 Red Hat, Inc.
  * Copyright (C) 2011 - 2012 Google, Inc.
+ * Copyright (C) 2016 Aleksander Morgado <aleksander@aleksander.es>
  */
 
 #include <config.h>
@@ -29,18 +30,14 @@
 
 #include "mm-base-modem-at.h"
 #include "mm-broadband-bearer-novatel-lte.h"
-#include "mm-log.h"
+#include "mm-log-object.h"
 #include "mm-modem-helpers.h"
 
-#define CONNECTION_CHECK_TIMEOUT_SEC 5
 #define QMISTATUS_TAG "$NWQMISTATUS:"
 
-G_DEFINE_TYPE (MMBroadbandBearerNovatelLte, mm_broadband_bearer_novatel_lte, MM_TYPE_BROADBAND_BEARER);
+G_DEFINE_TYPE (MMBroadbandBearerNovatelLte, mm_broadband_bearer_novatel_lte, MM_TYPE_BROADBAND_BEARER)
 
-struct _MMBroadbandBearerNovatelLtePrivate {
-    /* timeout id for checking whether we're still connected */
-    guint connection_poller;
-};
+/*****************************************************************************/
 
 static gchar *
 normalize_qmistatus (const gchar *status)
@@ -58,46 +55,6 @@ normalize_qmistatus (const gchar *status)
     return normalized_status;
 }
 
-/*****************************************************************************/
-/* 3GPP Connection sequence */
-
-typedef struct {
-    MMBroadbandBearerNovatelLte *self;
-    MMBaseModem *modem;
-    MMAtSerialPort *primary;
-    MMPort *data;
-    GCancellable *cancellable;
-    GSimpleAsyncResult *result;
-    gint retries;
-} DetailedConnectContext;
-
-static void
-detailed_connect_context_complete_and_free (DetailedConnectContext *ctx)
-{
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->result);
-    g_object_unref (ctx->cancellable);
-    if (ctx->data)
-        g_object_unref (ctx->data);
-    g_object_unref (ctx->primary);
-    g_object_unref (ctx->modem);
-    g_object_unref (ctx->self);
-    g_slice_free (DetailedConnectContext, ctx);
-}
-
-static MMBearerConnectResult *
-connect_3gpp_finish (MMBroadbandBearer *self,
-                     GAsyncResult *res,
-                     GError **error)
-{
-    if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error))
-        return NULL;
-
-    return mm_bearer_connect_result_ref (g_simple_async_result_get_op_res_gpointer (G_SIMPLE_ASYNC_RESULT (res)));
-}
-
-static gboolean connect_3gpp_qmistatus (DetailedConnectContext *ctx);
-
 static gboolean
 is_qmistatus_connected (const gchar *str)
 {
@@ -114,136 +71,210 @@ is_qmistatus_disconnected (const gchar *str)
     return g_strrstr (str, "QMI State: DISCONNECTED") || g_strrstr (str, "QMI State: QMI_WDS_PKT_DATA_DISCONNECTED");
 }
 
+static gboolean
+is_qmistatus_call_failed (const gchar *str)
+{
+    str = mm_strip_tag (str, QMISTATUS_TAG);
+
+    return (g_strrstr (str, "QMI_RESULT_FAILURE:QMI_ERR_CALL_FAILED") != NULL);
+}
+
+/*****************************************************************************/
+/* Connection status monitoring */
+
+static MMBearerConnectionStatus
+load_connection_status_finish (MMBaseBearer  *bearer,
+                               GAsyncResult  *res,
+                               GError       **error)
+{
+    GError *inner_error = NULL;
+    gssize value;
+
+    value = g_task_propagate_int (G_TASK (res), &inner_error);
+    if (inner_error) {
+        g_propagate_error (error, inner_error);
+        return MM_BEARER_CONNECTION_STATUS_UNKNOWN;
+    }
+    return (MMBearerConnectionStatus)value;
+}
+
 static void
-poll_connection_ready (MMBaseModem *modem,
+poll_connection_ready (MMBaseModem  *modem,
                        GAsyncResult *res,
-                       MMBroadbandBearerNovatelLte *bearer)
+                       GTask        *task)
 {
     const gchar *result;
     GError *error = NULL;
 
     result = mm_base_modem_at_command_finish (modem, res, &error);
-    if (!result) {
-        mm_warn ("QMI connection status failed: %s", error->message);
-        g_error_free (error);
-        return;
-    }
-
-    if (is_qmistatus_disconnected (result)) {
-        mm_bearer_report_disconnection (MM_BEARER (bearer));
-        g_source_remove (bearer->priv->connection_poller);
-        bearer->priv->connection_poller = 0;
-    }
+    if (!result)
+        g_task_return_error (task, error);
+    else if (is_qmistatus_disconnected (result))
+        g_task_return_int (task, MM_BEARER_CONNECTION_STATUS_DISCONNECTED);
+    else
+        g_task_return_int (task, MM_BEARER_CONNECTION_STATUS_CONNECTED);
+    g_object_unref (task);
 }
 
-static gboolean
-poll_connection (MMBroadbandBearerNovatelLte *bearer)
+static void
+load_connection_status (MMBaseBearer        *bearer,
+                        GAsyncReadyCallback  callback,
+                        gpointer             user_data)
 {
+    GTask       *task;
     MMBaseModem *modem = NULL;
 
-    g_object_get (MM_BEARER (bearer),
-                  MM_BEARER_MODEM, &modem,
+    task = g_task_new (bearer, NULL, callback, user_data);
+
+    g_object_get (MM_BASE_BEARER (bearer),
+                  MM_BASE_BEARER_MODEM, &modem,
                   NULL);
+
     mm_base_modem_at_command (
         modem,
         "$NWQMISTATUS",
         3,
         FALSE,
-        (GAsyncReadyCallback)poll_connection_ready,
-        bearer);
-    g_object_unref (modem);
+        (GAsyncReadyCallback) poll_connection_ready,
+        task);
 
-    return TRUE;
+    g_object_unref (modem);
 }
 
+/*****************************************************************************/
+/* 3GPP Connection sequence */
+
+typedef struct {
+    MMBaseModem *modem;
+    MMPortSerialAt *primary;
+    MMPort *data;
+    gint retries;
+} DetailedConnectContext;
+
+static void
+detailed_connect_context_free (DetailedConnectContext *ctx)
+{
+    if (ctx->data)
+        g_object_unref (ctx->data);
+    g_object_unref (ctx->primary);
+    g_object_unref (ctx->modem);
+    g_slice_free (DetailedConnectContext, ctx);
+}
+
+static MMBearerConnectResult *
+connect_3gpp_finish (MMBroadbandBearer *self,
+                     GAsyncResult *res,
+                     GError **error)
+{
+    return g_task_propagate_pointer (G_TASK (res), error);
+}
+
+static gboolean connect_3gpp_qmistatus (GTask *task);
+
 static void
 connect_3gpp_qmistatus_ready (MMBaseModem *modem,
                               GAsyncResult *res,
-                              DetailedConnectContext *ctx)
+                              GTask *task)
 {
+    MMBroadbandBearerNovatelLte *self;
+    DetailedConnectContext *ctx;
     const gchar *result;
     gchar *normalized_result;
     GError *error = NULL;
 
-    result = mm_base_modem_at_command_finish (MM_BASE_MODEM (ctx->modem),
-                                              res,
-                                              &error);
-    if (!result) {
-        mm_warn ("QMI connection status failed: %s", error->message);
-        if (!g_error_matches (error, MM_MOBILE_EQUIPMENT_ERROR, MM_MOBILE_EQUIPMENT_ERROR_UNKNOWN)) {
-            g_simple_async_result_take_error (ctx->result, error);
-            detailed_connect_context_complete_and_free (ctx);
-            return;
-        }
-        g_error_free (error);
-        result = "Unknown error";
-    } else if (is_qmistatus_connected (result)) {
-        MMBearerIpConfig *config;
+    self = g_task_get_source_object (task);
+    ctx  = g_task_get_task_data (task);
 
-        mm_dbg("Connected");
-        ctx->self->priv->connection_poller = g_timeout_add_seconds (CONNECTION_CHECK_TIMEOUT_SEC,
-                                                                    (GSourceFunc)poll_connection,
-                                                                    ctx->self);
-        config = mm_bearer_ip_config_new ();
-        mm_bearer_ip_config_set_method (config, MM_BEARER_IP_METHOD_DHCP);
-        g_simple_async_result_set_op_res_gpointer (
-            ctx->result,
-            mm_bearer_connect_result_new (ctx->data, config, NULL),
-            (GDestroyNotify)mm_bearer_connect_result_unref);
-        g_object_unref (config);
-        detailed_connect_context_complete_and_free (ctx);
+    if (g_task_return_error_if_cancelled (task)) {
+        g_object_unref (task);
         return;
     }
 
-    mm_dbg ("Error: '%s'", result);
+    result = mm_base_modem_at_command_full_finish (modem, res, &error);
+    if (!result) {
+        if (!g_error_matches (error, MM_MOBILE_EQUIPMENT_ERROR, MM_MOBILE_EQUIPMENT_ERROR_UNKNOWN)) {
+            g_task_return_error (task, error);
+            g_object_unref (task);
+            return;
+        }
+        mm_obj_dbg (self, "connection status failed: %s; will retry", error->message);
+        g_error_free (error);
+        goto retry;
+    }
+
+    if (is_qmistatus_connected (result)) {
+        MMBearerIpConfig *config;
+
+        mm_obj_dbg (self, "connected");
+        config = mm_bearer_ip_config_new ();
+        mm_bearer_ip_config_set_method (config, MM_BEARER_IP_METHOD_DHCP);
+        g_task_return_pointer (
+            task,
+            mm_bearer_connect_result_new (ctx->data, config, NULL),
+            (GDestroyNotify)mm_bearer_connect_result_unref);
+        g_object_unref (task);
+        g_object_unref (config);
+        return;
+    }
+
+    /* Don't retry if the call failed */
+    if (is_qmistatus_call_failed (result)) {
+        mm_obj_dbg (self, "not retrying: call failed");
+        ctx->retries = 0;
+    }
+
+retry:
     if (ctx->retries > 0) {
         ctx->retries--;
-        mm_dbg ("Retrying status check in a second. %d retries left.",
-                ctx->retries);
-        g_timeout_add_seconds (1, (GSourceFunc)connect_3gpp_qmistatus, ctx);
+        mm_obj_dbg (self, "retrying status check in a second: %d retries left", ctx->retries);
+        g_timeout_add_seconds (1, (GSourceFunc)connect_3gpp_qmistatus, task);
         return;
     }
 
     /* Already exhausted all retries */
     normalized_result = normalize_qmistatus (result);
-    g_simple_async_result_set_error (ctx->result,
-                                     MM_CORE_ERROR,
-                                     MM_CORE_ERROR_FAILED,
-                                     "QMI connect failed: %s",
-                                     normalized_result);
+    g_task_return_new_error (task,
+                             MM_CORE_ERROR,
+                             MM_CORE_ERROR_FAILED,
+                             "QMI connect failed: %s",
+                             normalized_result);
+    g_object_unref (task);
     g_free (normalized_result);
-    detailed_connect_context_complete_and_free (ctx);
 }
 
 static gboolean
-connect_3gpp_qmistatus (DetailedConnectContext *ctx)
+connect_3gpp_qmistatus (GTask *task)
 {
-    mm_base_modem_at_command (
+    DetailedConnectContext *ctx;
+
+    ctx = g_task_get_task_data (task);
+
+    mm_base_modem_at_command_full (
         ctx->modem,
+        ctx->primary,
         "$NWQMISTATUS",
         3, /* timeout */
         FALSE, /* allow_cached */
+        FALSE, /* is_raw */
+        g_task_get_cancellable (task),
         (GAsyncReadyCallback)connect_3gpp_qmistatus_ready, /* callback */
-        ctx); /* user_data */
+        task); /* user_data */
 
-    return FALSE;
+    return G_SOURCE_REMOVE;
 }
 
 static void
 connect_3gpp_qmiconnect_ready (MMBaseModem *modem,
                                GAsyncResult *res,
-                               DetailedConnectContext *ctx)
+                               GTask *task)
 {
     const gchar *result;
     GError *error = NULL;
 
-    result = mm_base_modem_at_command_finish (MM_BASE_MODEM (modem),
-                                              res,
-                                              &error);
+    result = mm_base_modem_at_command_full_finish (modem, res, &error);
     if (!result) {
-        mm_warn ("QMI connection failed: %s", error->message);
-        g_simple_async_result_take_error (ctx->result, error);
-        detailed_connect_context_complete_and_free (ctx);
+        g_task_return_error (task, error);
+        g_object_unref (task);
         return;
     }
 
@@ -253,121 +284,93 @@ connect_3gpp_qmiconnect_ready (MMBaseModem *modem,
      * happened. Instead, we need to poll the modem to see if it's
      * ready.
      */
-    g_timeout_add_seconds (1, (GSourceFunc)connect_3gpp_qmistatus, ctx);
+    g_timeout_add_seconds (1, (GSourceFunc)connect_3gpp_qmistatus, task);
 }
 
 static void
-connect_3gpp_authenticate (DetailedConnectContext *ctx)
+connect_3gpp_authenticate (GTask *task)
 {
+    MMBroadbandBearerNovatelLte *self;
+    DetailedConnectContext *ctx;
     MMBearerProperties *config;
     gchar *command, *apn, *user, *password;
 
-    config = mm_bearer_peek_config (MM_BEARER (ctx->self));
-    apn = mm_at_serial_port_quote_string (mm_bearer_properties_get_apn (config));
-    user = mm_at_serial_port_quote_string (mm_bearer_properties_get_user (config));
-    password = mm_at_serial_port_quote_string (mm_bearer_properties_get_password (config));
+    self = g_task_get_source_object (task);
+    ctx = g_task_get_task_data (task);
+
+    config = mm_base_bearer_peek_config (MM_BASE_BEARER (self));
+    apn = mm_port_serial_at_quote_string (mm_bearer_properties_get_apn (config));
+    user = mm_port_serial_at_quote_string (mm_bearer_properties_get_user (config));
+    password = mm_port_serial_at_quote_string (mm_bearer_properties_get_password (config));
     command = g_strdup_printf ("$NWQMICONNECT=,,,,,,%s,,,%s,%s",
                                apn, user, password);
     g_free (apn);
     g_free (user);
     g_free (password);
-    mm_base_modem_at_command (
+    mm_base_modem_at_command_full (
         ctx->modem,
+        ctx->primary,
         command,
         10, /* timeout */
         FALSE, /* allow_cached */
+        FALSE, /* is_raw */
+        g_task_get_cancellable (task),
         (GAsyncReadyCallback)connect_3gpp_qmiconnect_ready,
-        ctx); /* user_data */
+        task); /* user_data */
     g_free (command);
 }
 
 static void
 connect_3gpp (MMBroadbandBearer *self,
               MMBroadbandModem *modem,
-              MMAtSerialPort *primary,
-              MMAtSerialPort *secondary,
+              MMPortSerialAt *primary,
+              MMPortSerialAt *secondary,
               GCancellable *cancellable,
               GAsyncReadyCallback callback,
               gpointer user_data)
 {
     DetailedConnectContext *ctx;
+    GTask *task;
 
     ctx = g_slice_new0 (DetailedConnectContext);
-    ctx->self = g_object_ref (self);
     ctx->modem = MM_BASE_MODEM (g_object_ref (modem));
     ctx->primary = g_object_ref (primary);
-    ctx->cancellable = g_object_ref (cancellable);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             connect_3gpp);
-    ctx->retries = 60;
+    ctx->retries = MM_BASE_BEARER_DEFAULT_CONNECTION_TIMEOUT;
+
+    task = g_task_new (self, cancellable, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)detailed_connect_context_free);
 
     /* Get a 'net' data port */
-    ctx->data = mm_base_modem_get_best_data_port (MM_BASE_MODEM (modem),
-                                                  MM_PORT_TYPE_NET);
+    ctx->data = mm_base_modem_get_best_data_port (ctx->modem, MM_PORT_TYPE_NET);
     if (!ctx->data) {
-        g_simple_async_result_set_error (
-            ctx->result,
+        g_task_return_new_error (
+            task,
             MM_CORE_ERROR,
             MM_CORE_ERROR_CONNECTED,
             "Couldn't connect: no available net port available");
-        detailed_connect_context_complete_and_free (ctx);
+        g_object_unref (task);
         return;
     }
 
-    connect_3gpp_authenticate (ctx);
+    connect_3gpp_authenticate (task);
 }
 
 /*****************************************************************************/
 /* 3GPP Disonnection sequence */
 
 typedef struct {
-    MMBroadbandBearer *self;
     MMBaseModem *modem;
-    MMAtSerialPort *primary;
-    MMAtSerialPort *secondary;
+    MMPortSerialAt *primary;
     MMPort *data;
-    GSimpleAsyncResult *result;
     gint retries;
 } DetailedDisconnectContext;
 
-static DetailedDisconnectContext *
-detailed_disconnect_context_new (MMBroadbandBearer *self,
-                                 MMBroadbandModem *modem,
-                                 MMAtSerialPort *primary,
-                                 MMAtSerialPort *secondary,
-                                 MMPort *data,
-                                 GAsyncReadyCallback callback,
-                                 gpointer user_data)
-{
-    DetailedDisconnectContext *ctx;
-
-    ctx = g_new0 (DetailedDisconnectContext, 1);
-    ctx->self = g_object_ref (self);
-    ctx->modem = MM_BASE_MODEM (g_object_ref (modem));
-    ctx->primary = g_object_ref (primary);
-    ctx->secondary = (secondary ? g_object_ref (secondary) : NULL);
-    ctx->data = g_object_ref (data);
-    ctx->result = g_simple_async_result_new (G_OBJECT (self),
-                                             callback,
-                                             user_data,
-                                             detailed_disconnect_context_new);
-    ctx->retries = 60;
-    return ctx;
-}
-
 static void
-detailed_disconnect_context_complete_and_free (DetailedDisconnectContext *ctx)
+detailed_disconnect_context_free (DetailedDisconnectContext *ctx)
 {
-    g_simple_async_result_complete_in_idle (ctx->result);
-    g_object_unref (ctx->result);
     g_object_unref (ctx->data);
-    if (ctx->secondary)
-        g_object_unref (ctx->secondary);
     g_object_unref (ctx->primary);
     g_object_unref (ctx->modem);
-    g_object_unref (ctx->self);
     g_free (ctx);
 }
 
@@ -376,43 +379,46 @@ disconnect_3gpp_finish (MMBroadbandBearer *self,
                         GAsyncResult *res,
                         GError **error)
 {
-    return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (res), error);
+    return g_task_propagate_boolean (G_TASK (res), error);
 }
 
-static gboolean disconnect_3gpp_qmistatus (DetailedDisconnectContext *ctx);
+static gboolean disconnect_3gpp_qmistatus (GTask *task);
 
 static void
-disconnect_3gpp_status_ready (MMBaseModem *modem,
+disconnect_3gpp_status_ready (MMBaseModem  *modem,
                               GAsyncResult *res,
-                              DetailedDisconnectContext *ctx)
+                              GTask        *task)
 {
-    const gchar *result;
-    GError *error = NULL;
-    gboolean is_connected = FALSE;
+    MMBroadbandBearerNovatelLte *self;
+    DetailedDisconnectContext   *ctx;
+    const gchar                 *result;
+    GError                      *error = NULL;
+    gboolean                     is_connected = FALSE;
 
-    result = mm_base_modem_at_command_finish (MM_BASE_MODEM (modem),
-                                              res,
-                                              &error);
+    self = g_task_get_source_object (task);
+
+    result = mm_base_modem_at_command_full_finish (modem, res, &error);
     if (result) {
-        mm_dbg ("QMI connection status: %s", result);
+        mm_obj_dbg (self, "QMI connection status: %s", result);
         if (is_qmistatus_disconnected (result)) {
-            g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
-            detailed_disconnect_context_complete_and_free (ctx);
+            g_task_return_boolean (task, TRUE);
+            g_object_unref (task);
             return;
-        } else if (is_qmistatus_connected (result)) {
-            is_connected = TRUE;
         }
+        if (is_qmistatus_connected (result))
+            is_connected = TRUE;
     } else {
-        mm_dbg ("QMI connection status failed: %s", error->message);
+        mm_obj_dbg (self, "QMI connection status failed: %s", error->message);
         g_error_free (error);
         result = "Unknown error";
     }
 
+    ctx = g_task_get_task_data (task);
+
     if (ctx->retries > 0) {
         ctx->retries--;
-        mm_dbg ("Retrying status check in a second. %d retries left.",
-                ctx->retries);
-        g_timeout_add_seconds (1, (GSourceFunc)disconnect_3gpp_qmistatus, ctx);
+        mm_obj_dbg (self, "retrying status check in a second: %d retries left", ctx->retries);
+        g_timeout_add_seconds (1, (GSourceFunc)disconnect_3gpp_qmistatus, task);
         return;
     }
 
@@ -423,84 +429,95 @@ disconnect_3gpp_status_ready (MMBaseModem *modem,
         gchar *normalized_result;
 
         normalized_result = normalize_qmistatus (result);
-        g_simple_async_result_set_error (ctx->result,
-                                         MM_CORE_ERROR,
-                                         MM_CORE_ERROR_FAILED,
-                                         "QMI disconnect failed: %s",
-                                         normalized_result);
+        g_task_return_new_error (task,
+                                 MM_CORE_ERROR,
+                                 MM_CORE_ERROR_FAILED,
+                                 "QMI disconnect failed: %s",
+                                 normalized_result);
         g_free (normalized_result);
     } else
-        g_simple_async_result_set_op_res_gboolean (ctx->result, TRUE);
+        g_task_return_boolean (task, TRUE);
 
-    detailed_disconnect_context_complete_and_free (ctx);
+    g_object_unref (task);
 }
 
 static gboolean
-disconnect_3gpp_qmistatus (DetailedDisconnectContext *ctx)
+disconnect_3gpp_qmistatus (GTask *task)
 {
-    mm_base_modem_at_command (
+    DetailedDisconnectContext *ctx;
+
+    ctx = g_task_get_task_data (task);
+
+    mm_base_modem_at_command_full (
         ctx->modem,
+        ctx->primary,
         "$NWQMISTATUS",
         3, /* timeout */
         FALSE, /* allow_cached */
-        (GAsyncReadyCallback)disconnect_3gpp_status_ready, /* callback */
-        ctx); /* user_data */
-
-    return FALSE;
+        FALSE, /* is_raw */
+        NULL, /* cancellable */
+        (GAsyncReadyCallback)disconnect_3gpp_status_ready,
+        task); /* user_data */
+    return G_SOURCE_REMOVE;
 }
 
 
 static void
-disconnect_3gpp_check_status (MMBaseModem *modem,
+disconnect_3gpp_check_status (MMBaseModem  *modem,
                               GAsyncResult *res,
-                              DetailedDisconnectContext *ctx)
+                              GTask        *task)
 {
-    GError *error = NULL;
+    MMBroadbandBearerNovatelLte *self;
+    GError                      *error = NULL;
 
-    mm_base_modem_at_command_finish (MM_BASE_MODEM (modem),
-                                     res,
-                                     &error);
+    self = g_task_get_source_object (task);
+
+    mm_base_modem_at_command_full_finish (modem, res, &error);
     if (error) {
-        mm_dbg("Disconnection error: %s", error->message);
+        mm_obj_dbg (self, "disconnection error: %s", error->message);
         g_error_free (error);
     }
 
-    disconnect_3gpp_qmistatus (ctx);
+    disconnect_3gpp_qmistatus (task);
 }
 
 static void
 disconnect_3gpp (MMBroadbandBearer *self,
                  MMBroadbandModem *modem,
-                 MMAtSerialPort *primary,
-                 MMAtSerialPort *secondary,
+                 MMPortSerialAt *primary,
+                 MMPortSerialAt *secondary,
                  MMPort *data,
                  guint cid,
                  GAsyncReadyCallback callback,
                  gpointer user_data)
 {
     DetailedDisconnectContext *ctx;
-    MMBroadbandBearerNovatelLte *bearer = MM_BROADBAND_BEARER_NOVATEL_LTE (self);
+    GTask *task;
 
-    if (bearer->priv->connection_poller) {
-        g_source_remove (bearer->priv->connection_poller);
-        bearer->priv->connection_poller = 0;
-    }
+    ctx = g_new0 (DetailedDisconnectContext, 1);
+    ctx->modem = MM_BASE_MODEM (g_object_ref (modem));
+    ctx->primary = g_object_ref (primary);
+    ctx->data = g_object_ref (data);
+    ctx->retries = MM_BASE_BEARER_DEFAULT_DISCONNECTION_TIMEOUT;
 
-    ctx = detailed_disconnect_context_new (self, modem, primary, secondary,
-                                           data, callback, user_data);
+    task = g_task_new (self, NULL, callback, user_data);
+    g_task_set_task_data (task, ctx, (GDestroyNotify)detailed_disconnect_context_free);
 
-    mm_base_modem_at_command (
+    mm_base_modem_at_command_full (
         ctx->modem,
+        ctx->primary,
         "$NWQMIDISCONNECT",
         10, /* timeout */
         FALSE, /* allow_cached */
+        FALSE, /* is_raw */
+        NULL, /* cancellable */
         (GAsyncReadyCallback)disconnect_3gpp_check_status,
-        ctx); /* user_data */
+        task); /* user_data */
 }
 
 /*****************************************************************************/
 
-MMBearer *
+MMBaseBearer *
 mm_broadband_bearer_novatel_lte_new_finish (GAsyncResult *res,
                                             GError **error)
 {
@@ -515,9 +532,9 @@ mm_broadband_bearer_novatel_lte_new_finish (GAsyncResult *res,
         return NULL;
 
     /* Only export valid bearers */
-    mm_bearer_export (MM_BEARER (bearer));
+    mm_base_bearer_export (MM_BASE_BEARER (bearer));
 
-    return MM_BEARER (bearer);
+    return MM_BASE_BEARER (bearer);
 }
 
 void
@@ -533,41 +550,24 @@ mm_broadband_bearer_novatel_lte_new (MMBroadbandModemNovatelLte *modem,
         cancellable,
         callback,
         user_data,
-        MM_BEARER_MODEM, modem,
-        MM_BEARER_CONFIG, config,
+        MM_BASE_BEARER_MODEM, modem,
+        MM_BASE_BEARER_CONFIG, config,
         NULL);
 }
 
 static void
 mm_broadband_bearer_novatel_lte_init (MMBroadbandBearerNovatelLte *self)
 {
-    self->priv = G_TYPE_INSTANCE_GET_PRIVATE ((self),
-                                              MM_TYPE_BROADBAND_BEARER_NOVATEL_LTE,
-                                              MMBroadbandBearerNovatelLtePrivate);
-
-    self->priv->connection_poller = 0;
-}
-
-static void
-finalize (GObject *object)
-{
-    MMBroadbandBearerNovatelLte *self = MM_BROADBAND_BEARER_NOVATEL_LTE (object);
-
-    if (self->priv->connection_poller)
-        g_source_remove (self->priv->connection_poller);
-
-    G_OBJECT_CLASS (mm_broadband_bearer_novatel_lte_parent_class)->finalize (object);
 }
 
 static void
 mm_broadband_bearer_novatel_lte_class_init (MMBroadbandBearerNovatelLteClass *klass)
 {
-    GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    MMBaseBearerClass *base_bearer_class = MM_BASE_BEARER_CLASS (klass);
     MMBroadbandBearerClass *broadband_bearer_class = MM_BROADBAND_BEARER_CLASS (klass);
 
-    g_type_class_add_private (object_class, sizeof (MMBroadbandBearerNovatelLtePrivate));
-
-    object_class->finalize = finalize;
+    base_bearer_class->load_connection_status = load_connection_status;
+    base_bearer_class->load_connection_status_finish = load_connection_status_finish;
 
     broadband_bearer_class->connect_3gpp = connect_3gpp;
     broadband_bearer_class->connect_3gpp_finish = connect_3gpp_finish;
